@@ -36,6 +36,10 @@
 #include <KConfigGroup>
 #include <KDebug>
 
+// static
+Bar *GameEngine::m_bar_ptr = 0;
+
+
 GameEngine::GameEngine(MainWindow *mainWindow)
     : m_mainWindow(mainWindow), randomCounter(0)
 {
@@ -46,6 +50,8 @@ GameEngine::GameEngine(MainWindow *mainWindow)
     connect(&m_elapsedTimeTimer, SIGNAL(timeout()), SLOT(increaseElapsedTime()));
 
     m_cheatsEnabled = !qgetenv("KDE_DEBUG").isEmpty();
+
+    m_bar_ptr = &m_bar;
 }
 
 GameEngine::~GameEngine()
@@ -82,6 +88,11 @@ bool GameEngine::gameIsPaused() const
     return !m_elapsedTimeTimer.isActive();
 }
 
+const Bar &GameEngine::bar()
+{
+    return *m_bar_ptr;
+}
+
 void GameEngine::pause()
 {
     if (gameIsPaused()) return;
@@ -109,17 +120,16 @@ void GameEngine::resume()
         }
     }
     
-    int barPosition = m_bar.position().x() + m_bar.getRect().width()/2;
     hideMessage();
-    emit gameResumed(barPosition);
+    emit gameResumed();
 }
 
-void GameEngine::togglePause()
+void GameEngine::setGamePaused(bool paused)
 {
-    if (gameIsPaused()) {
-        resume();
-    } else {
+    if (paused) {
         pause();
+    } else {
+        resume();
     }
 }
 
@@ -134,25 +144,32 @@ void GameEngine::moveBar(int newPos)
     int y = m_bar.getRect().y();
     int x = newPos - w/2;
     
-    if (x < 0) x = 0;
-    else if (x > width - w) x = width - w;
+    if (x < 0) {
+        x = 0;
+        emit resetMousePosition();
+    }
+    else if (x > width - w) {
+        x = width - w;
+        emit resetMousePosition();
+    }
     
     m_bar.moveTo(x, y);
 }
 
 void GameEngine::moveBarLeft()
 {
-    moveBar(m_bar.getRect().left() + m_bar.getRect().width()/2 - BAR_MOVEMENT);
+    moveBar(m_bar.center() - BAR_MOVEMENT);
 }
 
 void GameEngine::moveBarRight()
 {
-    moveBar(m_bar.getRect().left() + m_bar.getRect().width()/2 + BAR_MOVEMENT);
+    moveBar(m_bar.center() + BAR_MOVEMENT);
 }
 
 void GameEngine::fire()
 {
     if (gameIsPaused()) {
+        kDebug() << "trying to fire while game is paused!!!";
         resume();
         return;
     }
@@ -163,7 +180,7 @@ void GameEngine::fire()
         ball->toBeFired = false;
         
         qreal ballCenter= ball->getRect().left() + (ball->getRect().width())/2;
-        qreal barCenter = m_bar.getRect().left() + (m_bar.getRect().width())/2;
+        qreal barCenter = m_bar.center();
         qreal angle = M_PI / 3;
         angle *= (barCenter - ballCenter) / (m_bar.getRect().width()/2);
         angle += M_PI_2;
@@ -180,14 +197,16 @@ void GameEngine::fire()
 
 void GameEngine::cheatSkipLevel() 
 {
-    if (m_cheatsEnabled)
+    if (m_cheatsEnabled) {
         loadNextLevel();
+    }
 }
 
 void GameEngine::cheatAddLife() 
 {
-    if (m_cheatsEnabled)
+    if (m_cheatsEnabled) {
         m_lives.append(new Life);
+    }
 }
 
 // TODO: external level loader???
@@ -289,7 +308,7 @@ void GameEngine::loadLevel()
     }
     
     m_balls.append(new Ball);
-    moveBar(m_bar.getRect().x() + m_bar.getRect().width()/2);
+    moveBar(m_bar.center());
     m_bar.reset();
     updateAttachedBalls();
     
@@ -415,7 +434,18 @@ void GameEngine::repaintMovingObjects()
         }
 
         // increase the speed a little
-        changeSpeed(AUTO_SPEED_INCREASE);
+        // if there is at least one ball moving
+        // and the game isn't paused
+        bool ballMoving = false;
+        foreach (Ball *ball, m_balls) {
+            if (!ball->toBeFired) {
+                ballMoving = true;
+                break;
+            }
+        }
+        if (ballMoving && !gameIsPaused()) {
+            changeSpeed(AUTO_SPEED_INCREASE);
+        }
     }
     
     // move attached balls if needed
@@ -460,7 +490,7 @@ void GameEngine::detectBallCollisions(Ball *ball)
     else if (m_bar.getRect().intersects(rect) && ball->directionY > 0) {
         
         qreal ballCenter = ball->getRect().left() + (ball->getRect().width())/2;
-        qreal barCenter  = m_bar.getRect().left() + (m_bar.getRect().width())/2;
+        qreal barCenter  = m_bar.center();
         
         if (ballCenter > m_bar.getRect().left() &&
                 ballCenter < m_bar.getRect().right()) {
@@ -655,5 +685,3 @@ inline void GameEngine::deleteAllObjects()
     while (!m_bricks.isEmpty()) delete m_bricks.takeFirst();
     while (!m_gifts.isEmpty()) delete m_gifts.takeFirst();
 }
-
-#include "gameengine.moc"
