@@ -14,6 +14,7 @@
 #include <QGraphicsSceneMouseEvent>
 #include <QPainter>
 
+#include <KAction>
 #include <KDebug>
 #include <KGameDifficulty>
 #include <KLocalizedString>
@@ -42,6 +43,7 @@ MainArea::MainArea()
 , m_paused(false)
 , m_pause_time(0)
 , m_penalty(0)
+, m_pause_action(0)
 {
     m_size = 500;
     QRect rect(0, 0, m_size, m_size);
@@ -62,9 +64,10 @@ MainArea::MainArea()
     {
         // draw gradient
         QPainter p(&tmp);
+        QColor color = palette().color(QPalette::Window);
         QLinearGradient grad(QPointF(0, 0), QPointF(0, height()));
-        grad.setColorAt(0, QColor(240, 240, 240));
-        grad.setColorAt(1, QColor(180, 180, 180));
+        grad.setColorAt(0, color.lighter(115));
+        grad.setColorAt(1, color.darker(115));
         p.fillRect(rect, grad);
     }
     m_background = QPixmap::fromImage(tmp);
@@ -94,7 +97,7 @@ void MainArea::updateSounds()
 
 Animation* MainArea::writeMessage(const QString& text)
 {
-    Message* message = new Message(text, m_msg_font);
+    Message* message = new Message(text, m_msg_font, m_size);
     message->setPosition(QPointF(m_size, m_size) / 2.0);
     addItem(message);
     message->setOpacityF(0.0);
@@ -123,7 +126,7 @@ Animation* MainArea::writeText(const QString& text, bool fade)
     m_welcome_msg.clear();
     foreach (const QString &line, text.split("\n")) {
         m_welcome_msg.append(
-            KSharedPtr<Message>(new Message(line, m_msg_font)));
+            KSharedPtr<Message>(new Message(line, m_msg_font, m_size)));
     }
     displayMessages(m_welcome_msg);
 
@@ -146,17 +149,21 @@ Animation* MainArea::writeText(const QString& text, bool fade)
 
 void MainArea::displayMessages(const QList<KSharedPtr<Message> >& messages)
 {
-    const int step = 45;
-    QPointF pos(m_size / 2.0, (m_size - step * messages.size()) / 2.0);
+    int totalHeight = 0;
+    foreach (KSharedPtr<Message> message, messages) {
+      totalHeight += message->height();
+    }
+    QPointF pos(m_size / 2.0, (m_size - totalHeight) / 2.0);
 
     for (int i = 0; i < messages.size(); i++) {
         KSharedPtr<Message> msg = messages[i];
+        int halfHeight = msg->height() / 2;
+        pos.ry() += halfHeight;
         msg->setPosition(pos);
         msg->setZValue(10.0);
         msg->show();
         addItem(msg.data());
-
-        pos.ry() += step;
+        pos.ry() += halfHeight;
     }
 }
 
@@ -180,7 +187,10 @@ void MainArea::togglePause()
     else {
         m_paused = true;
         m_timer.stop();
-        writeText(i18n("Game paused\nClick or press P to resume"), false);
+        QString shortcut = m_pause_action ?
+          m_pause_action->shortcut().toString() :
+          "P";
+        writeText(i18n("Game paused\nClick or press %1 to resume", shortcut), false);
 
         if(m_last_game_time >= 5) {
             m_penalty += 5000;
@@ -247,6 +257,11 @@ void MainArea::start()
     m_player.play(AudioPlayer::START);
 }
 
+void MainArea::setPauseAction(KAction* action)
+{
+  m_pause_action = action;
+}
+
 QPointF MainArea::randomPoint() const
 {
     double x = (double)rand() * (m_size - radius() * 2) / RAND_MAX + radius();
