@@ -78,10 +78,11 @@ void MainWindow::setupActions()
 {
     // Game
     KAction* abort = actionCollection()->addAction("game_abort");
-    abort->setText(i18n("Abort game"));
+    abort->setText(i18n("End game"));
     connect(abort, SIGNAL(triggered()), m_main, SLOT(abort()));
 
-    KStandardGameAction::pause(m_main, SLOT(togglePause()), actionCollection());
+    KAction* pause = KStandardGameAction::pause(m_main, SLOT(togglePause()), actionCollection());
+    m_main->setPauseAction(pause);
     KStandardGameAction::highscores(this, SLOT(highscores()), actionCollection());
     KStandardGameAction::quit(this, SLOT(close()), actionCollection());
 
