@@ -1,37 +1,52 @@
-/*---------------------------------------------------------------------------
-
-  freecell.cpp  implements a patience card game
-
-     Copyright 1997 Rodolfo Borges <barrett@labma.ufrj.br>
-     Copyright 2000 Stephan Kulow <coolo@kde.org>
-
- * Permission to use, copy, modify, and distribute this software and its
- * documentation for any purpose and without fee is hereby granted,
- * provided that the above copyright notice appear in all copies and that
- * both that copyright notice and this permission notice appear in
- * supporting documentation.
+/*
+ * Copyright (C) 1997 Rodolfo Borges <barrett@labma.ufrj.br>
+ * Copyright (C) 1998-2009 Stephan Kulow <coolo@kde.org>
  *
- * This file is provided AS IS with no warranties of any kind.  The author
- * shall have no liability with respect to the infringement of copyrights,
- * trade secrets or any patents by this file or any part thereof.  In no
- * event will the author be liable for any lost revenue or profits or
- * other special, indirect and consequential damages.
-
----------------------------------------------------------------------------*/
+ * License of original code:
+ * -------------------------------------------------------------------------
+ *   Permission to use, copy, modify, and distribute this software and its
+ *   documentation for any purpose and without fee is hereby granted,
+ *   provided that the above copyright notice appear in all copies and that
+ *   both that copyright notice and this permission notice appear in
+ *   supporting documentation.
+ *
+ *   This file is provided AS IS with no warranties of any kind.  The author
+ *   shall have no liability with respect to the infringement of copyrights,
+ *   trade secrets or any patents by this file or any part thereof.  In no
+ *   event will the author be liable for any lost revenue or profits or
+ *   other special, indirect and consequential damages.
+ * -------------------------------------------------------------------------
+ *
+ * License of modifications/additions made after 2009-01-01:
+ * -------------------------------------------------------------------------
+ *   This program is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU General Public License as
+ *   published by the Free Software Foundation; either version 2 of 
+ *   the License, or (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
+ *
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * -------------------------------------------------------------------------
+ */
 
 #include "freecell.h"
-#include "deck.h"
-#include "patsolve/freecell.h"
+
+#include "carddeck.h"
+#include "dealerinfo.h"
 #include "speeds.h"
+#include "patsolve/freecellsolver.h"
 
-#include <cstdlib>
-#include <cassert>
+#include <KDebug>
+#include <KLocale>
 
-#include <QTimer>
-#include <QList>
+#include <QtCore/QList>
+#include <QtCore/QTimer>
 
-#include <klocale.h>
-#include <kdebug.h>
 
 const int CHUNKSIZE = 100;
 
@@ -41,7 +56,7 @@ void FreecellPile::moveCards(CardList &c, Pile *to)
         Pile::moveCards(c, to);
         return;
     }
-    Freecell *b = dynamic_cast<Freecell*>(dscene());
+    Freecell *b = dynamic_cast<Freecell*>(scene());
     if (b) {
         b->moveCards(c, this, to);
     }
@@ -52,36 +67,38 @@ void FreecellPile::moveCards(CardList &c, Pile *to)
 Freecell::Freecell()
     : DealerScene()
 {
-    Deck::create_deck(this, 1);
-    Deck::deck()->hide();
+    setDeck( new CardDeck() );
+
+    const qreal topRowDist = 1.08;
+    const qreal bottomRowDist = 1.13;
+    const qreal targetOffsetDist = ( 7 * bottomRowDist + 1 ) - ( 3 * topRowDist + 1 );
 
-    kDebug(11111) << "cards" << Deck::deck()->cards().count();
     for (int i = 0; i < 8; i++)
     {
-        FreecellPile *p = new FreecellPile(1 + i, this);
+        FreecellPile *p = new FreecellPile(1 + i, QString( "store%1" ).arg( i ));
         store[i] = p;
-        store[i]->setPilePos(1 + 11.3 * i, 14 );
-        store[i]->setObjectName( QString( "store%1" ).arg( i ) );
+        p->setPilePos( bottomRowDist * i, 1.3 );
         p->setAddFlags(Pile::addSpread | Pile::several);
         p->setRemoveFlags(Pile::several);
         p->setCheckIndex(0);
-        p->setReservedSpace( QSizeF( 10, 35 ) );
+        p->setReservedSpace( QSizeF( 1.0, 3.5 ) );
+        addPile(p);
     }
 
     for (int i = 0; i < 4; i++)
     {
-        freecell[i] = new Pile (1 + 8 +i, this);
-        freecell[i]->setPilePos(1 + 10.8 * i, 1);
-        freecell[i]->setObjectName( QString( "freecell%1" ).arg( i ) );
+        freecell[i] = new Pile (1 + 8 + i, QString( "freecell%1" ).arg( i ));
+        freecell[i]->setPilePos(topRowDist * i, 0);
         freecell[i]->setType(Pile::FreeCell);
+        addPile(freecell[i]);
     }
 
     for (int i = 0; i < 4; i++)
     {
-        target[i] = new Pile(1 + 8 + 4 +i, this);
-        target[i]->setObjectName( QString( "target%1" ).arg( i ) );
-        target[i]->setPilePos(4 + 10.8 * ( 4 + i ), 1);
+        target[i] = new Pile(1 + 8 + 4 + i, QString( "target%1" ).arg( i ));
+        target[i]->setPilePos(targetOffsetDist + topRowDist * i, 0);
         target[i]->setType(Pile::KlondikeTarget);
+        addPile(target[i]);
     }
 
     setActions(DealerScene::Demo | DealerScene::Hint);
@@ -96,7 +113,8 @@ Freecell::~Freecell()
 
 void Freecell::restart()
 {
-    Deck::deck()->collectAndShuffle();
+    deck()->returnAllCards();
+    deck()->shuffle( gameNumber() );
     deal();
 }
 
@@ -113,12 +131,9 @@ void Freecell::countFreeCells(int &free_cells, int &free_stores) const
 
 void Freecell::moveCards(CardList &c, FreecellPile *from, Pile *to)
 {
-    assert(c.count() > 1);
-    setWaiting(true);
+    Q_ASSERT(c.count() > 1);
 
     from->moveCardsBack(c);
-    waitfor = c.first();
-    connect(waitfor, SIGNAL(stoped(Card*)), SLOT(waitForMoving(Card*)));
 
     PileList fcs;
 
@@ -131,7 +146,7 @@ void Freecell::moveCards(CardList &c, FreecellPile *from, Pile *to)
         if (store[i]->isEmpty() && to != store[i]) fss.append(store[i]);
 
     if (fcs.count() == 0) {
-        assert(fss.count());
+        Q_ASSERT(fss.count());
         fcs.append(fss.last());
         fss.erase(--fss.end());
     }
@@ -141,7 +156,9 @@ void Freecell::moveCards(CardList &c, FreecellPile *from, Pile *to)
     current_weight = 1000;
     movePileToPile(c, to, fss, fcs, 0, c.count(), 0);
 
-    if (!waitfor->animated())
+    if (deck()->hasAnimatedCards())
+        connect(deck(), SIGNAL(cardAnimationDone()), this, SLOT(waitForMoving()));
+    else
         QTimer::singleShot(0, this, SLOT(startMoving()));
 }
 
@@ -153,14 +170,14 @@ struct MoveAway {
 
 void Freecell::movePileToPile(CardList &c, Pile *to, PileList fss, PileList &fcs, int start, int count, int debug_level)
 {
-    kDebug(11111) << debug_level << "movePileToPile" << c.count() << " " << start  << " " << count;
+    kDebug() << debug_level << "movePileToPile" << c.count() << " " << start  << " " << count;
     int moveaway = 0;
     if (count > fcs.count() + 1) {
         moveaway = (fcs.count() + 1);
         while (moveaway * 2 < count)
             moveaway <<= 1;
     }
-    kDebug(11111) << debug_level << "moveaway" << moveaway;
+    kDebug() << debug_level << "moveaway" << moveaway;
 
     QList<MoveAway> moves_away;
 
@@ -168,7 +185,7 @@ void Freecell::movePileToPile(CardList &c, Pile *to, PileList fss, PileList &fcs
         moveaway = count - (fcs.count() + 1);
     }
     while (count > fcs.count() + 1) {
-        assert(fss.count());
+        Q_ASSERT(fss.count());
         MoveAway ma;
         ma.firstfree = fss[0];
         ma.start = start;
@@ -183,7 +200,7 @@ void Freecell::movePileToPile(CardList &c, Pile *to, PileList fss, PileList &fcs
             moveaway = count - (fcs.count() + 1);
     }
     int moving = qMin(count, qMin(c.count() - start, fcs.count() + 1));
-    assert(moving);
+    Q_ASSERT(moving);
 
     for (int i = 0; i < moving - 1; i++) {
         moves.append(new MoveHint(c[c.count() - i - 1 - start], fcs[i], current_weight));
@@ -211,26 +228,31 @@ void Freecell::movePileToPile(CardList &c, Pile *to, PileList fss, PileList &fcs
 
 void Freecell::startMoving()
 {
-    kDebug(11111) << "startMoving\n";
+    kDebug() << "startMoving\n";
     if (moves.isEmpty()) {
         takeState();
         return;
     }
 
-    setWaiting(true);
     MoveHint *mh = moves.first();
     moves.erase(moves.begin());
     CardList empty;
     empty.append(mh->card());
-    assert(mh->card() == mh->card()->source()->top());
-    assert(mh->pile()->legalAdd(empty));
+
+    Pile * p = mh->card()->source();
+    Q_ASSERT(mh->card() == p->top());
+    Q_ASSERT(mh->pile()->legalAdd(empty));
     mh->pile()->add(mh->card());
 
     int duration = qMax( DURATION_MOVEBACK * mh->priority() / 1000, 1 );
     mh->pile()->moveCardsBack(empty, duration );
-    waitfor = mh->card();
-    kDebug(11111) << "wait for moving end" << mh->card()->rank() << " " << mh->card()->suit() << mh->priority();
-    connect(mh->card(), SIGNAL(stoped(Card*)), SLOT(waitForMoving(Card*)));
+    p->layoutCards( duration );
+
+    if (deck()->hasAnimatedCards())
+        connect(deck(), SIGNAL(cardAnimationDone()), this, SLOT(waitForMoving()));
+    else
+        QTimer::singleShot(0, this, SLOT(startMoving()));
+
     delete mh;
 }
 
@@ -238,35 +260,36 @@ void Freecell::newDemoMove(Card *m)
 {
     DealerScene::newDemoMove(m);
     if (m != m->source()->top())
-        m->disconnect();
+        m->disconnect( this );
 }
 
-void Freecell::waitForMoving(Card *c)
+void Freecell::waitForMoving()
 {
-    if (waitfor != c)
-        return;
-    c->disconnect(this);
+    disconnect(deck(), SIGNAL(cardAnimationDone()), this, SLOT(waitForMoving()));
     startMoving();
-    setWaiting(false);
 }
 
-bool Freecell::cardDblClicked(Card *c)
+bool Freecell::cardDoubleClicked(Card *c)
 {
     // target move
-    if (DealerScene::cardDblClicked(c))
+    if (DealerScene::cardDoubleClicked(c))
         return true;
 
     if (c->animated())
         return false;
 
     if (c == c->source()->top() && c->realFace())
+    {
         for (int i = 0; i < 4; i++)
-            if (freecell[i]->isEmpty()) {
-                CardList empty;
-                empty.append(c);
-                c->source()->moveCards(empty, freecell[i]);
+        {
+            if (freecell[i]->isEmpty())
+            {
+                c->source()->moveCards(QList<Card*>() << c, freecell[i]);
+                onGameStateAlteredByUser();
                 return true;
             }
+        }
+    }
     return false;
 }
 
@@ -338,12 +361,10 @@ void Freecell::getHints()
     if ( demoActive() )
         return;
 
-    for (PileList::Iterator it = piles.begin(); it != piles.end(); ++it)
+    foreach ( Pile *store, piles() )
     {
-        Pile *store = *it;
         if (store->isEmpty())
             continue;
-//        kDebug(11111) << "trying" << store->top()->name();
 
         CardList cards = store->cards();
         while (cards.count() && !cards.first()->realFace()) cards.erase(cards.begin());
@@ -352,10 +373,8 @@ void Freecell::getHints()
         while (iti != cards.end())
         {
             if (store->legalRemove(*iti)) {
-//                kDebug(11111) << "could remove" << (*iti)->name();
-                for (PileList::Iterator pit = piles.begin(); pit != piles.end(); ++pit)
+                foreach (Pile *dest, piles())
                 {
-                    Pile *dest = *pit;
                     if (dest == store)
                         continue;
                     if (store->indexOf(*iti) == 0 && dest->isEmpty() && !dest->target())
@@ -391,21 +410,29 @@ void Freecell::getHints()
 void Freecell::deal()
 {
     int column = 0;
-    while (!Deck::deck()->isEmpty())
+    while (deck()->hasUndealtCards())
     {
-        Card *c = Deck::deck()->nextCard();
-        store[column]->add (c, false);
+        Card *c = deck()->takeCard();
+        addCardForDeal( store[column], c, true, store[0]->pos() );
         column = (column + 1) % 8;
     }
+    startDealAnimation();
 }
 
-static class LocalDealerInfo3 : public DealerInfo
+
+
+static class FreecellDealerInfo : public DealerInfo
 {
 public:
-    LocalDealerInfo3() : DealerInfo(I18N_NOOP("Freecell"), 3) {}
-    virtual DealerScene *createGame() { return new Freecell(); }
-} ldi8;
+    FreecellDealerInfo()
+      : DealerInfo(I18N_NOOP("Freecell"), FreecellId)
+    {}
+
+    virtual DealerScene *createGame() const
+    {
+        return new Freecell();
+    }
+} freecellDealerInfo;
 
-//-------------------------------------------------------------------------//
 
 #include "freecell.moc"
