@@ -1,27 +1,51 @@
 /*
- * Permission to use, copy, modify, and distribute this software and its
- * documentation for any purpose and without fee is hereby granted,
- * provided that the above copyright notice appear in all copies and that
- * both that copyright notice and this permission notice appear in
- * supporting documentation.
+ * Copyright (C) 1995 Paul Olav Tvete <paul@troll.no>
+ * Copyright (C) 2000-2009 Stephan Kulow <coolo@kde.org>
  *
- * This file is provided AS IS with no warranties of any kind.  The author
- * shall have no liability with respect to the infringement of copyrights,
- * trade secrets or any patents by this file or any part thereof.  In no
- * event will the author be liable for any lost revenue or profits or
- * other special, indirect and consequential damages.
-*/
+ * License of original code:
+ * -------------------------------------------------------------------------
+ *   Permission to use, copy, modify, and distribute this software and its
+ *   documentation for any purpose and without fee is hereby granted,
+ *   provided that the above copyright notice appear in all copies and that
+ *   both that copyright notice and this permission notice appear in
+ *   supporting documentation.
+ *
+ *   This file is provided AS IS with no warranties of any kind.  The author
+ *   shall have no liability with respect to the infringement of copyrights,
+ *   trade secrets or any patents by this file or any part thereof.  In no
+ *   event will the author be liable for any lost revenue or profits or
+ *   other special, indirect and consequential damages.
+ * -------------------------------------------------------------------------
+ *
+ * License of modifications/additions made after 2009-01-01:
+ * -------------------------------------------------------------------------
+ *   This program is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU General Public License as
+ *   published by the Free Software Foundation; either version 2 of 
+ *   the License, or (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
+ *
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * -------------------------------------------------------------------------
+ */
 
 #include "pile.h"
 
-#include "cardmaps.h"
-#include "dealer.h"
-#include "deck.h"
+#include "carddeck.h"
+#include "cardscene.h"
 #include "render.h"
-#include "speeds.h"
 
 #include <KDebug>
 
+#include <QtCore/QTimer>
+#include <QtCore/QPropertyAnimation>
+#include <QtGui/QPainter>
+
 #include <cmath>
 
 
@@ -37,39 +61,43 @@ const int Pile::autoTurnTop   = 0x0200;
 const int Pile::wholeColumn   = 0x0400;
 const int Pile::demoOK        = 0x0800;
 
-Pile::Pile( int _index, DealerScene* parent)
-    : QGraphicsPixmapItem( ),
+Pile::Pile( int _index, const QString & objectName )
+    : QGraphicsPixmapItem(),
+      removeFlags(0),
+      addFlags(0),
       _atype(Custom),
       _rtype(Custom),
+      _checkIndex(-1),
       myIndex(_index),
       _target(false),
-      m_highlighted( false ),
-      m_graphicVisible( true )
+      m_graphicVisible( true ),
+      m_highlighted( false )
 {
-    setObjectName( "<unknown>" );
-    parent->addItem( this );
-
-    // Make the patience aware of this pile.
-    dscene()->addPile(this);
+    if ( objectName.isEmpty() )
+        setObjectName( QString("pile%1").arg(_index) );
+    else
+        setObjectName( objectName );
 
     QGraphicsItem::setVisible(true); // default
-    _checkIndex = -1;
-    addFlags    = 0;
-    removeFlags = 0;
 
     setZValue(0);
-    setSpread( 3.3 );
-    setReservedSpace( QSizeF( 10, 10 ) );
+    setSpread( 0, 0.33 );
+    setReservedSpace( QSizeF( 1, 1 ) );
     setShapeMode( QGraphicsPixmapItem::BoundingRectShape );
     setMaximumSpace( QSizeF( 1, 1 ) ); // just to make it valid
     m_relayoutTimer = new QTimer( this );
     m_relayoutTimer->setSingleShot( true );
     connect( m_relayoutTimer, SIGNAL(timeout()), SLOT(relayoutCards()) );
+
+    m_fadeAnimation = new QPropertyAnimation( this, "highlightedness", this );
+    m_fadeAnimation->setDuration( DURATION_CARDHIGHLIGHT );
+    m_fadeAnimation->setKeyValueAt( 0, 0 );
+    m_fadeAnimation->setKeyValueAt( 1, 1 );
 }
 
-DealerScene *Pile::dscene() const
+CardScene *Pile::cardScene() const
 {
-    return dynamic_cast<DealerScene*>( scene() );
+    return dynamic_cast<CardScene*>( scene() );
 }
 
 void Pile::setType(PileType type)
@@ -117,7 +145,7 @@ void Pile::setRemoveType(PileType type)
 
 Pile::~Pile()
 {
-    dscene()->removePile(this);
+//     dscene()->removePile(this);
 
     for (CardList::Iterator it = m_cards.begin(); it != m_cards.end(); ++it)
     {
@@ -125,17 +153,16 @@ Pile::~Pile()
             int i = -13;
             if ((*it)->source())
                 i = (*it)->source()->index();
-            kDebug(11111) << "pile doesn't match" << index() << " - " << i;
+            kDebug() << "pile doesn't match" << index() << " - " << i;
         }
         (*it)->setSource(0);
     }
     delete m_relayoutTimer;
 }
 
-void Pile::setPilePos( double x,  double y )
+void Pile::setPilePos( qreal x,  qreal y )
 {
     _pilePos = QPointF( x, y );
-    rescale();
 }
 
 QPointF Pile::pilePos() const
@@ -143,38 +170,33 @@ QPointF Pile::pilePos() const
     return _pilePos;
 }
 
-void Pile::rescale()
+void Pile::updatePixmap()
 {
-    if (!scene())
+    if ( !scene() )
         return;
 
-    QPointF new_pos = QPointF( _pilePos.x() * cardMap::self()->cardWidth() / 10.,
-                               _pilePos.y() * cardMap::self()->cardHeight() / 10. );
-    //kDebug(11111) << scene()->sceneRect() << pos() << new_pos << dscene()->offsetX();
-    if ( new_pos.x() < 0 )
-        new_pos.setX( scene()->width() - cardMap::self()->cardWidth() + new_pos.x() );
-    else
-        new_pos.rx() += dscene()->offsetX();
-    if ( new_pos.y() < 0 )
-        new_pos.setY( scene()->height() - cardMap::self()->cardHeight() + new_pos.y() );
+    QSize size = cardScene()->deck()->cardSize();
+    QPixmap pix( size );
+    pix.fill( Qt::transparent );
 
-    if ( new_pos != pos() )
-    {
-        setPos( new_pos );
-        tryRelayoutCards();
-    }
-
-    QSize size = cardMap::self()->cardSize();
     if ( m_graphicVisible )
     {
-        setPixmap( Render::renderElement( isHighlighted() ? "pile_selected" : "pile", size ) );
-    }
-    else
-    {
-        QPixmap blank( size );
-        blank.fill( Qt::transparent );
-        setPixmap( blank );
+        if ( m_fadeAnimation->state() == QAbstractAnimation::Running )
+        {
+            QPainter p( &pix );
+            p.setOpacity( 1 - m_highlightedness );
+            p.drawPixmap( 0, 0, Render::renderElement( "pile", size ) );
+            p.setOpacity( m_highlightedness );
+            p.drawPixmap( 0, 0, Render::renderElement( "pile_selected", size ) );
+        }
+        else
+        {
+            QString id = m_highlighted ? "pile_selected" : "pile";
+            pix = Render::renderElement( id, cardScene()->deck()->cardSize() );
+        }
     }
+
+    setPixmap( pix );
 }
 
 bool Pile::legalAdd( const CardList& _cards, bool ) const
@@ -196,7 +218,7 @@ bool Pile::legalAdd( const CardList& _cards, bool ) const
 
     switch (addType()) {
         case Custom:
-            return dscene()->checkAdd( checkIndex(), this, _cards );
+            return cardScene()->checkAdd( checkIndex(), this, _cards );
             break;
         case KlondikeTarget:
             return add_klondikeTarget(_cards);
@@ -229,7 +251,7 @@ bool Pile::legalRemove(const Card *c, bool demo) const
 
     switch (removeType()) {
         case Custom:
-            return dscene()->checkRemove( checkIndex(), this, c);
+            return cardScene()->checkRemove( checkIndex(), this, c);
             break;
         case KlondikeTarget:
         case GypsyStore:
@@ -247,12 +269,12 @@ bool Pile::legalRemove(const Card *c, bool demo) const
 
 void Pile::setVisible(bool vis)
 {
+    if ( vis == isVisible() )
+        return;
+
     QGraphicsItem::setVisible(vis);
-
-    for (CardList::Iterator it = m_cards.begin(); it != m_cards.end(); ++it)
-        (*it)->setVisible(vis);
-
-    dscene()->relayoutPiles();
+    foreach (Card *c, m_cards)
+        c->setVisible(vis);
 }
 
 int Pile::indexOf(const Card *c) const
@@ -281,10 +303,8 @@ Card *Pile::top() const
 
 void Pile::clear()
 {
-    for (CardList::Iterator it = m_cards.begin(); it != m_cards.end(); ++it)
-    {
-        (*it)->setSource(0);
-    }
+    foreach ( Card *card, m_cards )
+        card->setSource(0);
     m_cards.clear();
 }
 
@@ -294,13 +314,13 @@ void Pile::relayoutCards()
 
     foreach ( Card * card, m_cards )
     {
-        if ( card->animated() || dscene()->isMoving( card ) ) {
-            tryRelayoutCards();
+        if ( card->animated() || cardScene()->cardsBeingDragged().contains( card ) ) {
+            m_relayoutTimer->start( 50 );
             return;
         }
     }
 
-    layoutCards( dscene()->speedUpTime( DURATION_RELAYOUT ) );
+    layoutCards( DURATION_RELAYOUT );
 }
 
 void Pile::add( Card *_card, int index)
@@ -308,18 +328,18 @@ void Pile::add( Card *_card, int index)
     if (_card->source() == this)
         return;
 
-    Pile *source = _card->source();
-    if (source) {
-        _card->setTakenDown(source->target() && !target());
-        source->remove(_card);
-        source->tryRelayoutCards();
-        _card->setVisible( isVisible() );
+    if (_card->scene() != scene())
+        scene()->addItem(_card);
+
+    Pile *oldSource = _card->source();
+    if (oldSource)
+    {
+        _card->setTakenDown(oldSource->target() && !target());
+        oldSource->remove(_card);
     }
 
     _card->setSource(this);
-
-    QSizeF offset = cardOffset( _card );
-    _card->setSpread( offset );
+    _card->setVisible( isVisible() );
 
     if (index == -1)
         m_cards.append(_card);
@@ -329,102 +349,53 @@ void Pile::add( Card *_card, int index)
         Q_ASSERT(m_cards[index] == 0);
         m_cards[index] = _card;
     }
-
-    tryRelayoutCards();
 }
 
 
 // Return the number of pixels in x and y that the card should be
 // offset from the start position of the pile.
-//
-// Note: Default is to only have vertical spread (Y direction).
-
-QSizeF Pile::cardOffset( const Card *card ) const
+QPointF Pile::cardOffset( const Card *card ) const
 {
     if ( addFlags & Pile::addSpread )
     {
-        if (card->realFace())
-            return QSizeF(0, spread());
-        else
-            return QSizeF(0, qMin( spread() * 0.6, 1.3) );
+        QPointF offset( spread().width() * cardScene()->deck()->cardWidth(),
+                        spread().height() * cardScene()->deck()->cardHeight() );
+        if (!card->realFace())
+            offset *= 0.6;
+        return offset;
     }
 
-    return QSize(0, 0);
+    return QPointF(0, 0);
 }
 
 /* override cardtype (for initial deal ) */
-void Pile::add( Card* _card, bool _facedown )
+void Pile::animatedAdd( Card* _card, bool faceUp )
 {
-    if (!_card)
-        return;
+    Q_ASSERT( _card );
 
-    // The top card
-    Card *t = top();
-
-    // If this pile is visible, then also show the card.
-    if (isVisible())
-        _card->show();
-    else
-        _card->hide();
-
-    _card->turn( !_facedown );
-
-    qreal x2, y2, z2;
-
-    if (t) {
-        // kDebug(11111) << "::add" << t->pos() << " " << t->spread() << " " << _card->name() << " " << t->name() << " " << _card->spread();
-        x2 = t->realX() + t->spread().width()  / 10 * cardMap::self()->cardWidth();
-        y2 = t->realY() + t->spread().height() / 10 * cardMap::self()->cardHeight();
-        z2 = t->realZ() + 1;
-    } else {
-        x2 = x();
-        y2 = y();
-        z2 = zValue() + 1;
-    }
-
-    Pile *source = _card->source();
+    if ( _card->source() )
+        _card->source()->relayoutCards();
 
+    QPointF origPos = _card->pos();
+    _card->turn( faceUp );
     add(_card);
+    layoutCards( DURATION_RELAYOUT);
 
-    bool face = _facedown;
+    _card->completeAnimation();
+    QPointF destPos = _card->pos();
+    _card->setPos( origPos );
 
-#if 0
-    if ( source == Deck::deck() ) // ignore then
-        face = false;
-#endif
-    if (face || !isVisible()) {
-        _card->setPos( QPointF( x2, y2 ) );
-        _card->setZValue( z2 );
-    } else {
-        if ( source == Deck::deck() && dscene()->isInitialDeal() )
-        {
-            _card->setPos(QPointF( x2, 0 - 3 * cardMap::self()->cardHeight() ) );
-        }
-        _card->setZValue( z2 );
-        qreal distx = x2 - _card->x();
-        qreal disty = y2 - _card->y();
-        qreal dist = sqrt( distx * distx + disty * disty );
-        qreal whole = sqrt( scene()->width() * scene()->width() + scene()->height() * scene()->height() );
-        _card->moveTo(x2, y2, z2, qRound( dist * 1000 / whole ) );
-
-        if ( _card->animated() )
-        {
-            dscene()->setWaiting( true );
-            connect(_card, SIGNAL(stopped(Card*)), SLOT(waitForMoving(Card*)));
-        }
-    }
-}
-
-void Pile::waitForMoving( Card*c )
-{
-    c->disconnect(this);
-    dscene()->setWaiting( false );
+    QPointF delta = destPos - _card->pos();
+    qreal dist = sqrt( delta.x() * delta.x() + delta.y() * delta.y() );
+    qreal whole = sqrt( scene()->width() * scene()->width() + scene()->height() * scene()->height() );
+    _card->moveTo(destPos, _card->zValue(), qRound( dist * DURATION_DEAL / whole ) );
 }
 
 void Pile::remove(Card *c)
 {
     Q_ASSERT(m_cards.contains(c));
     m_cards.removeAll(c);
+    c->setSource(0);
 }
 
 void Pile::hideCards( const CardList & cards )
@@ -439,14 +410,41 @@ void Pile::unhideCards( const CardList & cards )
         m_cards.append(*it);
 }
 
-void Pile::setHighlighted( bool flag ) {
-    m_highlighted = flag;
-    rescale();
+void Pile::setHighlighted( bool flag )
+{
+    if ( flag != m_highlighted )
+    {
+        m_highlighted = flag;
+
+        m_fadeAnimation->setDirection( flag
+                                       ? QAbstractAnimation::Forward
+                                       : QAbstractAnimation::Backward );
+
+        if ( m_fadeAnimation->state() != QAbstractAnimation::Running )
+            m_fadeAnimation->start();
+    }
+}
+
+bool Pile::isHighlighted() const
+{
+    return m_highlighted;
+}
+
+void Pile::setHighlightedness( qreal highlightedness )
+{
+    m_highlightedness = highlightedness;
+    updatePixmap();
+    return;
+}
+
+qreal Pile::highlightedness() const
+{
+    return m_highlightedness;
 }
 
 void Pile::setGraphicVisible( bool flag ) {
     m_graphicVisible = flag;
-    rescale();
+    updatePixmap();
 }
 
 
@@ -454,7 +452,6 @@ CardList Pile::cardPressed(Card *c)
 {
     CardList result;
 
-    //kDebug(11111) << gettime() << "cardPressed" << c->name() << " " << c->isFaceUp();
     emit pressed( c );
 
     if ( !legalRemove(c) )
@@ -472,7 +469,7 @@ CardList Pile::cardPressed(Card *c)
         }
         if (below >= 0) {
             (*it)->stopAnimation();
-            (*it)->setZValue(128 + below);
+            (*it)->setZValue(1000 + below);
             below++;
             result.append(*it);
         }
@@ -482,19 +479,23 @@ CardList Pile::cardPressed(Card *c)
 
 void Pile::moveCards(CardList &cl, Pile *to)
 {
-    if (!cl.count())
+    if (cl.isEmpty())
         return;
 
-    for (CardList::Iterator it = cl.begin(); it != cl.end(); ++it)
-        to->add(*it);
-
-    if (removeFlags & autoTurnTop && top()) {
-        Card *t = top();
-        if (!t->isFaceUp()) {
-            t->flipTo(t->x(), t->y(), DURATION_FLIP );
-        }
+    foreach (Card * c, cl)
+    {
+        Q_ASSERT( c->source() == this );
+        to->add(c);
     }
 
+    Card *t = top();
+    if (t && !t->isFaceUp() && removeFlags & autoTurnTop)
+    {
+        t->animate(t->pos(), t->zValue(), 1, 0, true, false, DURATION_FLIP);
+    }
+
+    relayoutCards();
+
     to->moveCardsBack(cl);
 }
 
@@ -506,6 +507,9 @@ void Pile::moveCardsBack(CardList &cl, int duration)
     if ( duration == -1 )
         duration = DURATION_MOVEBACK;
 
+    foreach ( Card *c, cl )
+        c->raise();
+
     layoutCards( duration );
 }
 
@@ -515,46 +519,41 @@ bool Pile::cardClicked(Card *c)
     return false;
 }
 
-bool Pile::cardDblClicked(Card *c)
+bool Pile::cardDoubleClicked(Card *c)
 {
-    emit dblClicked(c);
+    emit doubleClicked(c);
     return false;
 }
 
-void Pile::tryRelayoutCards()
-{
-    int delay = dscene()->speedUpTime( 40 );
-    if ( delay < 30 )
-        m_relayoutTimer->start( 400 );
-    else
-        m_relayoutTimer->start( 40 );
-}
-
 void Pile::layoutCards(int duration)
 {
-    QSizeF preferredSize( 0, 0 );
-    foreach ( const Card * card, m_cards )
-    {
-        // The spreads should hopefully have all one sign, or we get in trouble
-        preferredSize.rwidth() += fabs( card->spread().width() );
-        preferredSize.rheight() += fabs( card->spread().height() );
-    }
+    if ( m_cards.isEmpty() )
+        return;
+
+    const QSize cardSize = cardScene()->deck()->cardSize();
+
+    QPointF totalOffset( 0, 0 );
+    for ( int i = 0; i < m_cards.size() - 1; ++i )
+        totalOffset += cardOffset( m_cards[i] );
+
+    qreal divx = 1;
+    if ( totalOffset.x() )
+        divx = qMin<qreal>( ( maximumSpace().width() - cardSize.width() ) / qAbs( totalOffset.x() ), 1.0 );
 
     qreal divy = 1;
-    if ( preferredSize.height() )
-        divy = qMin<qreal>( ( maximumSpace().height() - cardMap::self()->cardHeight() ) / preferredSize.height() * 10 / cardMap::self()->cardHeight(), 1. );
-    qreal divx = 1;
-    if ( preferredSize.width() )
-        divx = qMin<qreal>( ( maximumSpace().width() - cardMap::self()->cardWidth() ) / preferredSize.width() * 10 / cardMap::self()->cardWidth(), 1. );
+    if ( totalOffset.y() )
+        divy = qMin<qreal>( ( maximumSpace().height() - cardSize.height() ) / qAbs( totalOffset.y() ), 1.0 );
 
     QPointF cardPos = pos();
     qreal z = zValue() + 1;
     foreach ( Card * card, m_cards )
     {
-        card->moveTo( cardPos.x(), cardPos.y(), z, dscene()->speedUpTime( duration ) );
-        cardPos.rx() += divx * card->spread().width() / 10 * cardMap::self()->cardWidth();
-        cardPos.ry() += divy * card->spread().height() / 10 * cardMap::self()->cardHeight();
-        z += 1;
+        card->animate( cardPos, z, 1, 0, card->isFaceUp(), false, duration );
+
+        QPointF offset = cardOffset( card );
+        cardPos.rx() += divx * offset.x();
+        cardPos.ry() += divy * offset.y();
+        ++z;
     }
 }
 
