@@ -33,6 +33,7 @@
 #include <kconfigdialog.h>
 #include <kdebug.h>
 #include <kglobal.h>
+#include <kicon.h>
 #include <klineedit.h>
 #include <klocale.h>
 #include <kmenu.h>
@@ -44,6 +45,7 @@
 #include <kstandarddirs.h>
 #include <kstandardguiitem.h>
 #include <kstatusbar.h>
+#include <ktoggleaction.h>
 
 #include <QTimer>
 
@@ -86,13 +88,6 @@ App::App(QWidget *parent)
 }
 
 
-/**
- * There are four areas in the status bar:
- * - game tip
- * - timer
- * - tile count
- * - cheat mode
- */
 void App::setupStatusBar()
 {
     m_gameTipLabel = new QLabel(i18n("Select a tile"), statusBar());
@@ -123,6 +118,11 @@ void App::setupActions()
     KStandardGameAction::redo(this, SLOT(redo()), actionCollection());
     KStandardGameAction::hint(this, SLOT(hint()), actionCollection());
 
+    KToggleAction *soundAction = new KToggleAction(KIcon("speaker"), i18n("Play Sounds"), this);
+    soundAction->setChecked(Prefs::sounds());
+    actionCollection()->addAction("sounds", soundAction);
+    connect(soundAction, SIGNAL(triggered(bool)), m_board, SLOT(setSoundsEnabled(bool)));
+
     // Settings
     KStandardAction::preferences(this, SLOT(showSettings()), actionCollection());
 
@@ -139,14 +139,9 @@ void App::setupActions()
     timer->start(1000);
 
     connect(m_board, SIGNAL(changed()), this, SLOT(updateTileDisplay()));
-
     connect(m_board, SIGNAL(endOfGame()), this, SLOT(slotEndOfGame()));
-
 }
 
-/**
- * Flags set in the previously played game should be reset.
- */
 void App::newGame()
 {
     m_board->newGame();
@@ -155,12 +150,6 @@ void App::newGame()
     updateItems();
 }
 
-/**
- * Currently this is done by undoing all moves done by the user as yet and
- * resetting the move history and the timer.
- * This might change over time. However, just make sure, the user gets his
- * currently played game as if it was started by the New Game action.
- */
 void App::restartGame()
 {
     m_board->setUpdatesEnabled(false);
@@ -177,26 +166,17 @@ void App::restartGame()
     updateItems();
 }
 
-
 void App::togglePause()
 {
     m_board->setPauseEnabled(!m_board->isPaused());
 }
 
-/**
- * If the game is paused, do not show the board and disable actions like undo
- * and such.
- */
 void App::setPauseEnabled(bool enabled)
 {
     m_board->setPauseEnabled(enabled);
     updateItems();
 }
 
-/**
- * The Undo action should set the cheat flag, so the user cannot end up in
- * the highscore dialog by making bad decisions. :)
- */
 void App::undo()
 {
     if (!m_board->canUndo()) {
@@ -221,10 +201,6 @@ void App::redo()
     updateItems();
 }
 
-/**
- * The Hint action should set the cheat flag, so the user cannot end up in
- * the highscore dialog by having been told what to do. :)
- */
 void App::hint()
 {
 #ifdef DEBUGGING
@@ -236,10 +212,6 @@ void App::hint()
     updateItems();
 }
 
-/**
- * According to the current state of the game (game over, pause ...) some
- * actions might better be disabled. This is the place to do so.
- */
 void App::updateItems()
 {
     if (m_board->isOver()) {
@@ -297,7 +269,7 @@ void App::slotEndOfGame()
                 scoreDialog.setComment(message);
                 scoreDialog.exec();
             } else {
-                QString message = i18nc("%1 - time string like hh:mm:ss", "\nYou made it in %1", timeString);
+                QString message = i18nc("%1 - time string like hh:mm:ss", "You made it in %1", timeString);
                 KMessageBox::information(this, message, i18n("End of Game"));
             }
         }
@@ -310,6 +282,7 @@ void App::updateTimeDisplay()
     if (m_board->isStuck() || m_board->isOver()) {
         return;
     }
+    //kDebug() << "Time: " << m_board->currentTime();
     int currentTime = m_board->currentTime();
     QString message = i18n("Your time: %1:%2:%3 %4",
                             QString().sprintf("%02d", currentTime / 3600),
@@ -324,7 +297,7 @@ void App::updateTimeDisplay()
 
 void App::updateTileDisplay()
 {
-    kDebug() << "updateTileDisplay()" << m_board->tilesLeft();
+    //kDebug() << "Tiles: " << m_board->tilesLeft();
     int numberOfTiles = (m_board->xTiles() * m_board->yTiles());
     QString message = i18n("Removed: %1/%2 ",
                     QString().sprintf("%d", numberOfTiles - m_board->tilesLeft()),
