@@ -1,7 +1,7 @@
 /*
     This file is part of the KDE games library
     Copyright (C) 2001 Andreas Beckermann (b_mann@gmx.de)
-    Copyright (C) 2001 Martin Heni (martin@heni-online.de)
+    Copyright (C) 2001 Martin Heni (kde at heni-online.de)
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -14,8 +14,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 // NAMING
@@ -28,12 +28,10 @@
 #ifndef __KGAMEDIALOGCONFIG_H__
 #define __KGAMEDIALOGCONFIG_H__
 
-#include <qwidget.h>
-#include <kdemacros.h>
+#include <QtGui/QWidget>
+#include <libkdegames_export.h>
 
-class QGridLayout;
-class QVBoxLayout;
-class QListBoxItem;
+class QListWidgetItem;
 
 class KGame;
 class KPlayer;
@@ -41,6 +39,8 @@ class KGamePropertyBase;
 
 class KGameDialogConfigPrivate;
 /**
+ * \class KGameDialogConfig kgamedialogconfig.h <KGameDialogConfig>
+ * 
  * Base class for configuration widgets.
  *
  * You can inherit from this and implement @ref submitToKGame, @ref
@@ -48,7 +48,7 @@ class KGameDialogConfigPrivate;
  * @short Base class for configuration widgets
  * @author Andreas Beckermann <b_mann@gmx.de>
  **/
-class KDE_EXPORT KGameDialogConfig : public QWidget
+class KDEGAMES_EXPORT KGameDialogConfig : public QWidget
 {
 	Q_OBJECT
 public:
@@ -129,19 +129,20 @@ public:
 protected:
 
 private:
-	KGameDialogConfigPrivate* d;
+	KGameDialogConfigPrivate* const d;
 };
 
+class KGameDialogGeneralConfigPrivate;
 /**
+ * \class KGameDialogGeneralConfig kgamedialogconfig.h <KGameDialogConfig>
+ * 
  * The main game configuration widget.
  * 
  * It currently contains a line edit for the name of the player only. You can
  * add widgets by using the KGameDialogGeneralConfig as parent parameter as it
- * uses @ref QLayout::autoAdd == true.
- * @shot The main @ref KGameDialog configuration widget
+ * uses QLayout::autoAdd == true.
  * @author Andreas Beckermann <b_mann@gmx.de>
  **/
-class KGameDialogGeneralConfigPrivate;
 class KGameDialogGeneralConfig : public KGameDialogConfig
 {
 	Q_OBJECT
@@ -150,16 +151,18 @@ public:
 	 * Construct a KGameDialogGeneralConfig. Currently it contains a line
 	 * edit widget to change the player name only.
 	 *
+	 * If you just want to add more widgets you can just create your widgets
+	 * with the KGameDialogGeneralConfig as parent as it uses
+	 * QLayout::setAutoAdd(true).
+	 *
+	 * @param parent Parent widget for this dialog.
 	 * @param initializeGUI If you really don't want to use the 
 	 * predefined widget and/or layout use FALSE here. Note that then none
 	 * of the predefined widgets (currently only the name of the player) 
 	 * will exist anymore.
 	 *
-	 * If you just want to add more widgets you can just create your widgets
-	 * with the KGameDialogGeneralConfig as parent as it uses @ref
-	 * QLayout::setAutoAdd(true).
 	 **/
-	KGameDialogGeneralConfig(QWidget* parent = 0, bool initializeGUI = true);
+        explicit KGameDialogGeneralConfig(QWidget* parent = 0, bool initializeGUI = true);
 	virtual ~KGameDialogGeneralConfig();
 
 	/**
@@ -194,7 +197,7 @@ public:
 	 **/
 	virtual void setAdmin(bool admin);
 
-protected slots:
+protected Q_SLOTS:
 	void slotPropertyChanged(KGamePropertyBase*, KPlayer*);
 
 protected:
@@ -207,7 +210,10 @@ private:
 };
 
 class KGameDialogNetworkConfigPrivate;
-class KDE_EXPORT KGameDialogNetworkConfig : public KGameDialogConfig
+/**
+ * \class KGameDialogNetworkConfig kgamedialogconfig.h <KGameDialogConfig>
+ */
+class KDEGAMES_EXPORT KGameDialogNetworkConfig : public KGameDialogConfig
 {
 	Q_OBJECT
 public:
@@ -236,6 +242,7 @@ public:
 	 * You are strongly encouraged to change at least the port!
 	 * @param port The default port to connect to / listen on
 	 * @param host The default host to connect to
+	 * @param server The default state. 0 For a server game, 1 to join a game
 	 **/
 	void setDefaultNetworkInfo(const QString& host, unsigned short int port,bool server=true);
 	
@@ -245,24 +252,23 @@ public:
 	 * @param name Game name. Important only for server mode. If not
 	 * set hostname will be used. In case of name conflict -2, -3 and so on will be added to name.
 	 * @param type Service type (something like _kwin4._tcp). It should be unique for application.
-	 * @since 3.4
 	 **/
-	void setDiscoveryInfo(const QString& type, const QString& name=QString::null);
+	void setDiscoveryInfo(const QString& type, const QString& name=QString());
 	
-signals:
+Q_SIGNALS:
   /**
   * This signal is emmited if the user changes the server type (client/server)
   * in the network configuration dialog. 
   *
-  * @param t - type type (0/1) of the connection
+  * @param t type (0/1) of the connection
   **/
-  void signalServerTypeChanged(int);
+  void signalServerTypeChanged(int t);
 
 
 protected:
 	void setConnected(bool connected, bool master = false);
 
-protected slots:
+protected Q_SLOTS:
 	void slotInitConnection();
 	void slotExitConnection();
 	void slotConnectionBroken();
@@ -273,6 +279,9 @@ private:
 };
 
 class KGameDialogMsgServerConfigPrivate;
+/**
+ * \class KGameDialogMsgServerConfig kgamedialogconfig.h <KGameDialogConfig>
+ */
 class KGameDialogMsgServerConfig : public KGameDialogConfig
 {
 	Q_OBJECT
@@ -280,20 +289,20 @@ public:
 	KGameDialogMsgServerConfig(QWidget* parent = 0);
 	virtual ~KGameDialogMsgServerConfig();
 
-	virtual void submitToKGame(KGame*, KPlayer*) {}
+	virtual void submitToKGame(KGame* g, KPlayer* p) { Q_UNUSED(g); Q_UNUSED(p); }
 
 	void setHasMsgServer(bool);
 
 	virtual void setKGame(KGame* g);
-	virtual void setAdmin(bool);
+	virtual void setAdmin(bool admin);
 
-protected slots:
+protected Q_SLOTS:
 	void slotChangeMaxClients();
 	void slotChangeAdmin();
 	void slotRemoveClient();
 
 protected:
-	void removeClient(Q_UINT32 id);
+	void removeClient(quint32 id);
 
 private:
 	KGameDialogMsgServerConfigPrivate* d;
@@ -301,6 +310,8 @@ private:
 
 class KGameDialogChatConfigPrivate;
 /**
+ * \class KGameDialogChatConfig kgamedialogconfig.h <KGameDialogConfig>
+ * 
  * This is not really a configuration widget but rather a simple chat widget.
  * This widget does nothing but just providing a @ref KGameChat object.
  * @short A chat widget inside a @ref KGameDialog
@@ -310,23 +321,25 @@ class KGameDialogChatConfig : public KGameDialogConfig
 {
 	Q_OBJECT
 public:
-	KGameDialogChatConfig(int chatMsgId, QWidget* parent = 0);
+	explicit KGameDialogChatConfig(int chatMsgId, QWidget* parent = 0);
 	virtual ~KGameDialogChatConfig();
 
 	virtual void setKGame(KGame* g);
 	virtual void setOwner(KPlayer* p);
 
-	virtual void submitToKGame(KGame*, KPlayer*) { }
+	virtual void submitToKGame(KGame* g, KPlayer* p) { Q_UNUSED(g); Q_UNUSED(p); }
 
 private:
 	KGameDialogChatConfigPrivate* d;
 };
 
+class KGameDialogConnectionConfigPrivate;
 /**
+ * \class KGameDialogConnectionConfig kgamedialogconfig.h <KGameDialogConfig>
+ * 
  * @short Lists all connected players and gives the ability to kick them off the
  * game
  **/
-class KGameDialogConnectionConfigPrivate;
 class KGameDialogConnectionConfig : public KGameDialogConfig
 {
 	Q_OBJECT
@@ -334,20 +347,21 @@ public:
 	KGameDialogConnectionConfig(QWidget* parent = 0);
 	virtual ~KGameDialogConnectionConfig();
 
-	virtual void setKGame(KGame*);
-	virtual void setOwner(KPlayer*);
+	virtual void setKGame(KGame* g);
+	virtual void setOwner(KPlayer* p);
 	virtual void setAdmin(bool admin);
 
-	virtual void submitToKGame(KGame*, KPlayer*) { }
+	virtual void submitToKGame(KGame* g, KPlayer* p) { Q_UNUSED(g); Q_UNUSED(p); }
 
 protected:
 	/**
-	 * @return The @ref QListBoxItem that belongs to the player p
+	 * @param p A player
+	 * @return The @ref QListWidgetItem that belongs to the player @p p
 	 **/
-	QListBoxItem* item(KPlayer* p) const;
+	QListWidgetItem* item(KPlayer* p) const;
 
-protected slots:
-	void slotKickPlayerOut(QListBoxItem* item);
+protected Q_SLOTS:
+	void slotKickPlayerOut(QListWidgetItem* item);
 	void slotPropertyChanged(KGamePropertyBase* prop, KPlayer* p);
 	void slotPlayerLeftGame(KPlayer* p);
 	void slotPlayerJoinedGame(KPlayer* p);
